/****************************************************************************
** Meta object code from reading C++ file 'package_manager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../libs/linglong/src/linglong/package_manager/package_manager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'package_manager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8linglong7service14PackageManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto linglong::service::PackageManager::qt_create_metaobjectdata<qt_meta_tag_ZN8linglong7service14PackageManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "linglong::service::PackageManager",
        "D-Bus Interface",
        "org.deepin.linglong.PackageManager1",
        "TaskAdded",
        "",
        "QDBusObjectPath",
        "object_path",
        "TaskRemoved",
        "RequestInteraction",
        "messageID",
        "QVariantMap",
        "additionalMessage",
        "SearchFinished",
        "jobID",
        "result",
        "PruneFinished",
        "ReplyReceived",
        "replies",
        "Configuration"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'TaskAdded'
        QtMocHelpers::SignalData<void(QDBusObjectPath)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'TaskRemoved'
        QtMocHelpers::SignalData<void(QDBusObjectPath)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'RequestInteraction'
        QtMocHelpers::SignalData<void(QDBusObjectPath, int, QVariantMap)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Int, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'SearchFinished'
        QtMocHelpers::SignalData<void(QString, QVariantMap)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 10, 14 },
        }}),
        // Signal 'PruneFinished'
        QtMocHelpers::SignalData<void(QString, QVariantMap)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 10, 14 },
        }}),
        // Signal 'ReplyReceived'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Configuration'
        QtMocHelpers::PropertyData<QVariantMap>(18, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<PackageManager, qt_meta_tag_ZN8linglong7service14PackageManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject linglong::service::PackageManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service14PackageManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service14PackageManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8linglong7service14PackageManagerE_t>.metaTypes,
    nullptr
} };

void linglong::service::PackageManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PackageManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->TaskAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 1: _t->TaskRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 2: _t->RequestInteraction((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3]))); break;
        case 3: _t->SearchFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 4: _t->PruneFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 5: _t->ReplyReceived((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(QDBusObjectPath )>(_a, &PackageManager::TaskAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(QDBusObjectPath )>(_a, &PackageManager::TaskRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(QDBusObjectPath , int , QVariantMap )>(_a, &PackageManager::RequestInteraction, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(QString , QVariantMap )>(_a, &PackageManager::SearchFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(QString , QVariantMap )>(_a, &PackageManager::PruneFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageManager::*)(const QVariantMap & )>(_a, &PackageManager::ReplyReceived, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantMap*>(_v) = _t->getConfiguration(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setConfiguration(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *linglong::service::PackageManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *linglong::service::PackageManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service14PackageManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int linglong::service::PackageManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void linglong::service::PackageManager::TaskAdded(QDBusObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void linglong::service::PackageManager::TaskRemoved(QDBusObjectPath _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void linglong::service::PackageManager::RequestInteraction(QDBusObjectPath _t1, int _t2, QVariantMap _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void linglong::service::PackageManager::SearchFinished(QString _t1, QVariantMap _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void linglong::service::PackageManager::PruneFinished(QString _t1, QVariantMap _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void linglong::service::PackageManager::ReplyReceived(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
