// Thish file is generated by /tools/codegen
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     Windows.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "ocppi/runtime/config/types/helper.hpp"

#include "ocppi/runtime/config/types/WindowsDevice.hpp"
#include "ocppi/runtime/config/types/Hyperv.hpp"
#include "ocppi/runtime/config/types/WindowsNetwork.hpp"
#include "ocppi/runtime/config/types/WindowsResources.hpp"

namespace ocppi {
namespace runtime {
namespace config {
namespace types {
/**
* Windows platform-specific configurations
*/

using nlohmann::json;

/**
* Windows platform-specific configurations
*/
struct Windows {
std::optional<std::map<std::string, nlohmann::json>> credentialSpec;
std::optional<std::vector<WindowsDevice>> devices;
std::optional<Hyperv> hyperv;
std::optional<bool> ignoreFlushesDuringBoot;
std::vector<std::string> layerFolders;
std::optional<WindowsNetwork> network;
std::optional<WindowsResources> resources;
std::optional<bool> servicing;
};
}
}
}
}

// clang-format on
