// Thish file is generated by /tools/codegen
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     Generators.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "ocppi/types/helper.hpp"

#include "ocppi/types/ContainerListItem.hpp"

namespace ocppi {
namespace types {
void from_json(const json & j, ContainerListItem & x);
void to_json(json & j, const ContainerListItem & x);

inline void from_json(const json & j, ContainerListItem& x) {
x.bundle = j.at("bundle").get<std::string>();
x.created = j.at("created").get<std::string>();
x.id = j.at("id").get<std::string>();
x.owner = j.at("owner").get<std::string>();
x.pid = j.at("pid").get<int64_t>();
x.status = j.at("status").get<std::string>();
}

inline void to_json(json & j, const ContainerListItem & x) {
j = json::object();
j["bundle"] = x.bundle;
j["created"] = x.created;
j["id"] = x.id;
j["owner"] = x.owner;
j["pid"] = x.pid;
j["status"] = x.status;
}

}
}

// clang-format on
