/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.PackageManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "src/linglong/adaptors/package_manager/gen_org_deepin_linglong_packagemanager1.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class OrgDeepinLinglongPackagemanager1Adaptor
 */

OrgDeepinLinglongPackagemanager1Adaptor::OrgDeepinLinglongPackagemanager1Adaptor(linglong::service::PackageManager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

OrgDeepinLinglongPackagemanager1Adaptor::~OrgDeepinLinglongPackagemanager1Adaptor()
{
    // destructor
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::configuration() const
{
    // get the value of property Configuration
    return qvariant_cast< QVariantMap >(parent()->property("Configuration"));
}

void OrgDeepinLinglongPackagemanager1Adaptor::setConfiguration(const QVariantMap &value)
{
    // set the value of property Configuration
    parent()->setProperty("Configuration", QVariant::fromValue(value));
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::Install(const QVariantMap &parameters)
{
    // handle method call org.deepin.linglong.PackageManager1.Install
    return parent()->Install(parameters);
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::InstallFromFile(const QDBusUnixFileDescriptor &fd, const QString &fileType, const QVariantMap &options)
{
    // handle method call org.deepin.linglong.PackageManager1.InstallFromFile
    return parent()->InstallFromFile(fd, fileType, options);
}

void OrgDeepinLinglongPackagemanager1Adaptor::Permissions()
{
    // handle method call org.deepin.linglong.PackageManager1.Permissions
    parent()->Permissions();
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::Prune()
{
    // handle method call org.deepin.linglong.PackageManager1.Prune
    return parent()->Prune();
}

void OrgDeepinLinglongPackagemanager1Adaptor::ReplyInteraction(const QDBusObjectPath &task, const QVariantMap &replies)
{
    // handle method call org.deepin.linglong.PackageManager1.ReplyInteraction
    parent()->ReplyInteraction(task, replies);
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::Search(const QVariantMap &parameters)
{
    // handle method call org.deepin.linglong.PackageManager1.Search
    return parent()->Search(parameters);
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::Uninstall(const QVariantMap &parameters)
{
    // handle method call org.deepin.linglong.PackageManager1.Uninstall
    return parent()->Uninstall(parameters);
}

QVariantMap OrgDeepinLinglongPackagemanager1Adaptor::Update(const QVariantMap &parameters)
{
    // handle method call org.deepin.linglong.PackageManager1.Update
    return parent()->Update(parameters);
}


#include "moc_gen_org_deepin_linglong_packagemanager1.cpp"
