// This file is generated by tools/codegen.sh
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     UabMetaInfo.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "linglong/api/types/v1/helper.hpp"

#include "linglong/api/types/v1/UabLayer.hpp"
#include "linglong/api/types/v1/Sections.hpp"

namespace linglong {
namespace api {
namespace types {
namespace v1 {
enum class Version : int;
}
}
}
}

namespace linglong {
namespace api {
namespace types {
namespace v1 {
using nlohmann::json;

struct UabMetaInfo {
/**
* The digest of the bundle section.
*/
std::string digest;
std::vector<UabLayer> layers;
/**
* whether this UAB file has been exported in only-App mode.
*/
std::optional<bool> onlyApp;
Sections sections;
/**
* The version 4 uuid of this UAB file, generated by UAB builder when this UAB file is
* created.
*/
std::string uuid;
/**
* The format version of linglong.meta data.
*/
Version version;
};
}
}
}
}

// clang-format on
