// This file is autogenerated. Changes will be overwritten.
#include "TXJY7IDDLQ/moc_cli.cpp"
#include "TXJY7IDDLQ/moc_dbus_notifier.cpp"
#include "SGLX75OABA/moc_properties_forwarder.cpp"
#include "UROKDND3XR/moc_initialize.cpp"
#include "FB3JYSBHC6/moc_package_manager.cpp"
#include "FB3JYSBHC6/moc_package_task.cpp"
#include "IUHSAB4IOT/moc_ostree_repo.cpp"
#include "52EBWSDMFU/moc_container_builder.cpp"
