/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.PackageManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef GEN_ORG_DEEPIN_LINGLONG_PACKAGEMANAGER1_H
#define GEN_ORG_DEEPIN_LINGLONG_PACKAGEMANAGER1_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.linglong.PackageManager1
 */
class OrgDeepinLinglongPackageManager1Interface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.linglong.PackageManager1"; }

public:
    OrgDeepinLinglongPackageManager1Interface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgDeepinLinglongPackageManager1Interface();

    Q_PROPERTY(QVariantMap Configuration READ configuration WRITE setConfiguration)
    inline QVariantMap configuration() const
    { return qvariant_cast< QVariantMap >(property("Configuration")); }
    inline void setConfiguration(const QVariantMap &value)
    { setProperty("Configuration", QVariant::fromValue(value)); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QVariantMap> Install(const QVariantMap &parameters)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(parameters);
        return asyncCallWithArgumentList(QStringLiteral("Install"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> InstallFromFile(const QDBusUnixFileDescriptor &fd, const QString &fileType, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(fd) << QVariant::fromValue(fileType) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("InstallFromFile"), argumentList);
    }

    inline QDBusPendingReply<> Permissions()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Permissions"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> Prune()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Prune"), argumentList);
    }

    inline QDBusPendingReply<> ReplyInteraction(const QDBusObjectPath &task, const QVariantMap &replies)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(task) << QVariant::fromValue(replies);
        return asyncCallWithArgumentList(QStringLiteral("ReplyInteraction"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> Search(const QVariantMap &parameters)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(parameters);
        return asyncCallWithArgumentList(QStringLiteral("Search"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> Uninstall(const QVariantMap &parameters)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(parameters);
        return asyncCallWithArgumentList(QStringLiteral("Uninstall"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> Update(const QVariantMap &parameters)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(parameters);
        return asyncCallWithArgumentList(QStringLiteral("Update"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void PruneFinished(const QString &taskID, const QVariantMap &result);
    void RequestInteraction(const QDBusObjectPath &task, int messageID, const QVariantMap &additionalMessage);
    void SearchFinished(const QString &taskID, const QVariantMap &result);
    void TaskAdded(const QDBusObjectPath &task);
    void TaskRemoved(const QDBusObjectPath &task);
};

namespace org {
  namespace deepin {
    namespace linglong {
      using PackageManager1 = ::OrgDeepinLinglongPackageManager1Interface;
    }
  }
}

#endif
