// Thish file is generated by /tools/codegen
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     State.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "ocppi/runtime/state/types/helper.hpp"

namespace ocppi {
namespace runtime {
namespace state {
namespace types {
enum class Status : int;
}
}
}
}

namespace ocppi {
namespace runtime {
namespace state {
namespace types {
/**
* Open Container Runtime State Schema
*/

using nlohmann::json;

/**
* Open Container Runtime State Schema
*/
struct State {
std::optional<std::map<std::string, std::string>> annotations;
std::string bundle;
/**
* the container's ID
*/
std::string id;
std::string ociVersion;
std::optional<int64_t> pid;
Status status;
};
}
}
}
}

// clang-format on
