/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.PackageManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef GEN_ORG_DEEPIN_LINGLONG_PACKAGEMANAGER1_H
#define GEN_ORG_DEEPIN_LINGLONG_PACKAGEMANAGER1_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "linglong/package_manager/package_manager.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.deepin.linglong.PackageManager1
 */
class OrgDeepinLinglongPackagemanager1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.deepin.linglong.PackageManager1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.deepin.linglong.PackageManager1\">\n"
"    <method name=\"InstallFromFile\">\n"
"      <arg direction=\"in\" type=\"h\" name=\"fd\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"fileType\"/>\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"options\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In2\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Install\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"parameters\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Uninstall\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"parameters\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Update\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"parameters\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Search\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"parameters\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Prune\">\n"
"      <annotation value=\"Remove unused base or runtime.\" name=\"org.freedesktop.DBus.Description\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Permissions\">\n"
"      <annotation value=\"Used by the client to check whether it has permission to call privileged methods\" name=\"org.freedesktop.DBus.Description\"/>\n"
"    </method>\n"
"    <signal name=\"SearchFinished\">\n"
"      <arg type=\"s\" name=\"taskID\"/>\n"
"      <arg type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"    </signal>\n"
"    <signal name=\"PruneFinished\">\n"
"      <arg type=\"s\" name=\"taskID\"/>\n"
"      <arg type=\"a{sv}\" name=\"result\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"    </signal>\n"
"    <method name=\"ReplyInteraction\">\n"
"      <arg type=\"o\" name=\"task\"/>\n"
"      <arg type=\"a{sv}\" name=\"replies\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\"/>\n"
"    </method>\n"
"    <property access=\"readwrite\" type=\"a{sv}\" name=\"Configuration\">\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n"
"    </property>\n"
"    <signal name=\"TaskAdded\">\n"
"      <arg type=\"o\" name=\"task\"/>\n"
"    </signal>\n"
"    <signal name=\"TaskRemoved\">\n"
"      <arg type=\"o\" name=\"task\"/>\n"
"    </signal>\n"
"    <signal name=\"RequestInteraction\">\n"
"      <arg type=\"o\" name=\"task\"/>\n"
"      <arg type=\"i\" name=\"messageID\"/>\n"
"      <arg type=\"a{sv}\" name=\"additionalMessage\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out2\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    OrgDeepinLinglongPackagemanager1Adaptor(linglong::service::PackageManager *parent);
    ~OrgDeepinLinglongPackagemanager1Adaptor() override;

    inline linglong::service::PackageManager *parent() const
    { return static_cast<linglong::service::PackageManager *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QVariantMap Configuration READ configuration WRITE setConfiguration)
    QVariantMap configuration() const;
    void setConfiguration(const QVariantMap &value);

public Q_SLOTS: // METHODS
    QVariantMap Install(const QVariantMap &parameters);
    QVariantMap InstallFromFile(const QDBusUnixFileDescriptor &fd, const QString &fileType, const QVariantMap &options);
    void Permissions();
    QVariantMap Prune();
    void ReplyInteraction(const QDBusObjectPath &task, const QVariantMap &replies);
    QVariantMap Search(const QVariantMap &parameters);
    QVariantMap Uninstall(const QVariantMap &parameters);
    QVariantMap Update(const QVariantMap &parameters);
Q_SIGNALS: // SIGNALS
    void PruneFinished(const QString &taskID, const QVariantMap &result);
    void RequestInteraction(const QDBusObjectPath &task, int messageID, const QVariantMap &additionalMessage);
    void SearchFinished(const QString &taskID, const QVariantMap &result);
    void TaskAdded(const QDBusObjectPath &task);
    void TaskRemoved(const QDBusObjectPath &task);
};

#endif
