// SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#pragma once

#include "linglong/api/dbus/v1/gen_org_deepin_linglong_task1.h"

namespace linglong::api::dbus::v1 {
using Task1 = OrgDeepinLinglongTask1Interface;
}
