/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.Task1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef GEN_ORG_DEEPIN_LINGLONG_TASK1_H
#define GEN_ORG_DEEPIN_LINGLONG_TASK1_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "linglong/package_manager/package_task.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.deepin.linglong.Task1
 */
class OrgDeepinLinglongTask1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.deepin.linglong.Task1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.deepin.linglong.Task1\">\n"
"    <method name=\"Cancel\"/>\n"
"    <property access=\"read\" type=\"i\" name=\"State\"/>\n"
"    <property access=\"read\" type=\"d\" name=\"Percentage\"/>\n"
"    <property access=\"read\" type=\"s\" name=\"Message\"/>\n"
"    <property access=\"read\" type=\"i\" name=\"Code\"/>\n"
"  </interface>\n"
        "")
public:
    OrgDeepinLinglongTask1Adaptor(linglong::service::PackageTask *parent);
    ~OrgDeepinLinglongTask1Adaptor() override;

    inline linglong::service::PackageTask *parent() const
    { return static_cast<linglong::service::PackageTask *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(int Code READ code)
    int code() const;

    Q_PROPERTY(QString Message READ message)
    QString message() const;

    Q_PROPERTY(double Percentage READ percentage)
    double percentage() const;

    Q_PROPERTY(int State READ state)
    int state() const;

public Q_SLOTS: // METHODS
    void Cancel();
Q_SIGNALS: // SIGNALS
};

#endif
