/****************************************************************************
** Meta object code from reading C++ file 'gen_org_deepin_linglong_packagemanager1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "gen_org_deepin_linglong_packagemanager1.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gen_org_deepin_linglong_packagemanager1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgDeepinLinglongPackagemanager1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgDeepinLinglongPackagemanager1Adaptor",
        "D-Bus Interface",
        "org.deepin.linglong.PackageManager1",
        "D-Bus Introspection",
        "  <interface name=\"org.deepin.linglong.PackageManager1\">\n    <m"
        "ethod name=\"InstallFromFile\">\n      <arg direction=\"in\" type="
        "\"h\" name=\"fd\"/>\n      <arg direction=\"in\" type=\"s\" name=\""
        "fileType\"/>\n      <arg direction=\"in\" type=\"a{sv}\" name=\"op"
        "tions\"/>\n      <annotation value=\"QVariantMap\" name=\"org.qtpr"
        "oject.QtDBus.QtTypeName.In2\"/>\n      <arg direction=\"out\" type"
        "=\"a{sv}\" name=\"result\"/>\n      <annotation value=\"QVariantMa"
        "p\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>"
        "\n    <method name=\"Install\">\n      <arg direction=\"in\" type="
        "\"a{sv}\" name=\"parameters\"/>\n      <annotation value=\"QVarian"
        "tMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg "
        "direction=\"out\" type=\"a{sv}\" name=\"result\"/>\n      <annotat"
        "ion value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.O"
        "ut0\"/>\n    </method>\n    <method name=\"Uninstall\">\n      <ar"
        "g direction=\"in\" type=\"a{sv}\" name=\"parameters\"/>\n      <an"
        "notation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeN"
        "ame.In0\"/>\n      <arg direction=\"out\" type=\"a{sv}\" name=\"re"
        "sult\"/>\n      <annotation value=\"QVariantMap\" name=\"org.qtpro"
        "ject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n    <method name=\""
        "Update\">\n      <arg direction=\"in\" type=\"a{sv}\" name=\"param"
        "eters\"/>\n      <annotation value=\"QVariantMap\" name=\"org.qtpr"
        "oject.QtDBus.QtTypeName.In0\"/>\n      <arg direction=\"out\" type"
        "=\"a{sv}\" name=\"result\"/>\n      <annotation value=\"QVariantMa"
        "p\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>"
        "\n    <method name=\"Search\">\n      <arg direction=\"in\" type=\""
        "a{sv}\" name=\"parameters\"/>\n      <annotation value=\"QVariantM"
        "ap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n      <arg di"
        "rection=\"out\" type=\"a{sv}\" name=\"result\"/>\n      <annotatio"
        "n value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out"
        "0\"/>\n    </method>\n    <method name=\"Prune\">\n      <annotati"
        "on value=\"Remove unused base or runtime.\" name=\"org.freedesktop"
        ".DBus.Description\"/>\n      <arg direction=\"out\" type=\"a{sv}\""
        " name=\"result\"/>\n      <annotation value=\"QVariantMap\" name=\""
        "org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n    <meth"
        "od name=\"Permissions\">\n      <annotation value=\"Used by the cl"
        "ient to check whether it has permission to call privileged methods"
        "\" name=\"org.freedesktop.DBus.Description\"/>\n    </method>\n   "
        " <signal name=\"SearchFinished\">\n      <arg type=\"s\" name=\"ta"
        "skID\"/>\n      <arg type=\"a{sv}\" name=\"result\"/>\n      <anno"
        "tation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeNam"
        "e.Out1\"/>\n    </signal>\n    <signal name=\"PruneFinished\">\n  "
        "    <arg type=\"s\" name=\"taskID\"/>\n      <arg type=\"a{sv}\" n"
        "ame=\"result\"/>\n      <annotation value=\"QVariantMap\" name=\"o"
        "rg.qtproject.QtDBus.QtTypeName.Out1\"/>\n    </signal>\n    <metho"
        "d name=\"ReplyInteraction\">\n      <arg type=\"o\" name=\"task\"/"
        ">\n      <arg type=\"a{sv}\" name=\"replies\"/>\n      <annotation"
        " value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In1\""
        "/>\n    </method>\n    <property access=\"readwrite\" type=\"a{sv}"
        "\" name=\"Configuration\">\n      <annotation value=\"QVariantMap\""
        " name=\"org.qtproject.QtDBus.QtTypeName\"/>\n    </property>\n    "
        "<signal name=\"TaskAdded\">\n      <arg type=\"o\" name=\"task\"/>"
        "\n    </signal>\n    <signal name=\"TaskRemoved\">\n      <arg typ"
        "e=\"o\" name=\"task\"/>\n    </signal>\n    <signal name=\"Request"
        "Interaction\">\n      <arg type=\"o\" name=\"task\"/>\n      <arg "
        "type=\"i\" name=\"messageID\"/>\n      <arg type=\"a{sv}\" name=\""
        "additionalMessage\"/>\n      <annotation value=\"QVariantMap\" nam"
        "e=\"org.qtproject.QtDBus.QtTypeName.Out2\"/>\n    </signal>\n  </i"
        "nterface>\n",
        "PruneFinished",
        "",
        "taskID",
        "QVariantMap",
        "result",
        "RequestInteraction",
        "QDBusObjectPath",
        "task",
        "messageID",
        "additionalMessage",
        "SearchFinished",
        "TaskAdded",
        "TaskRemoved",
        "Install",
        "parameters",
        "InstallFromFile",
        "QDBusUnixFileDescriptor",
        "fd",
        "fileType",
        "options",
        "Permissions",
        "Prune",
        "ReplyInteraction",
        "replies",
        "Search",
        "Uninstall",
        "Update",
        "Configuration"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PruneFinished'
        QtMocHelpers::SignalData<void(const QString &, const QVariantMap &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'RequestInteraction'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, int, const QVariantMap &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 }, { 0x80000000 | 8, 14 },
        }}),
        // Signal 'SearchFinished'
        QtMocHelpers::SignalData<void(const QString &, const QVariantMap &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'TaskAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'TaskRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'Install'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(18, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 8, 19 },
        }}),
        // Slot 'InstallFromFile'
        QtMocHelpers::SlotData<QVariantMap(const QDBusUnixFileDescriptor &, const QString &, const QVariantMap &)>(20, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 23 }, { 0x80000000 | 8, 24 },
        }}),
        // Slot 'Permissions'
        QtMocHelpers::SlotData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Prune'
        QtMocHelpers::SlotData<QVariantMap()>(26, 6, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'ReplyInteraction'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &, const QVariantMap &)>(27, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 8, 28 },
        }}),
        // Slot 'Search'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(29, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 8, 19 },
        }}),
        // Slot 'Uninstall'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(30, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 8, 19 },
        }}),
        // Slot 'Update'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(31, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 8, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Configuration'
        QtMocHelpers::PropertyData<QVariantMap>(32, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OrgDeepinLinglongPackagemanager1Adaptor, qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OrgDeepinLinglongPackagemanager1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>.metaTypes,
    nullptr
} };

void OrgDeepinLinglongPackagemanager1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgDeepinLinglongPackagemanager1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PruneFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 1: _t->RequestInteraction((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3]))); break;
        case 2: _t->SearchFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 3: _t->TaskAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 4: _t->TaskRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 5: { QVariantMap _r = _t->Install((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 6: { QVariantMap _r = _t->InstallFromFile((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->Permissions(); break;
        case 8: { QVariantMap _r = _t->Prune();
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->ReplyInteraction((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 10: { QVariantMap _r = _t->Search((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 11: { QVariantMap _r = _t->Uninstall((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 12: { QVariantMap _r = _t->Update((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackagemanager1Adaptor::*)(const QString & , const QVariantMap & )>(_a, &OrgDeepinLinglongPackagemanager1Adaptor::PruneFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackagemanager1Adaptor::*)(const QDBusObjectPath & , int , const QVariantMap & )>(_a, &OrgDeepinLinglongPackagemanager1Adaptor::RequestInteraction, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackagemanager1Adaptor::*)(const QString & , const QVariantMap & )>(_a, &OrgDeepinLinglongPackagemanager1Adaptor::SearchFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackagemanager1Adaptor::*)(const QDBusObjectPath & )>(_a, &OrgDeepinLinglongPackagemanager1Adaptor::TaskAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackagemanager1Adaptor::*)(const QDBusObjectPath & )>(_a, &OrgDeepinLinglongPackagemanager1Adaptor::TaskRemoved, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantMap*>(_v) = _t->configuration(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setConfiguration(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OrgDeepinLinglongPackagemanager1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinLinglongPackagemanager1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39OrgDeepinLinglongPackagemanager1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int OrgDeepinLinglongPackagemanager1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OrgDeepinLinglongPackagemanager1Adaptor::PruneFinished(const QString & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgDeepinLinglongPackagemanager1Adaptor::RequestInteraction(const QDBusObjectPath & _t1, int _t2, const QVariantMap & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void OrgDeepinLinglongPackagemanager1Adaptor::SearchFinished(const QString & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgDeepinLinglongPackagemanager1Adaptor::TaskAdded(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgDeepinLinglongPackagemanager1Adaptor::TaskRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
