/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.Task1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "src/linglong/adaptors/task/gen_org_deepin_linglong_task1.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class OrgDeepinLinglongTask1Adaptor
 */

OrgDeepinLinglongTask1Adaptor::OrgDeepinLinglongTask1Adaptor(linglong::service::PackageTask *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

OrgDeepinLinglongTask1Adaptor::~OrgDeepinLinglongTask1Adaptor()
{
    // destructor
}

int OrgDeepinLinglongTask1Adaptor::code() const
{
    // get the value of property Code
    return qvariant_cast< int >(parent()->property("Code"));
}

QString OrgDeepinLinglongTask1Adaptor::message() const
{
    // get the value of property Message
    return qvariant_cast< QString >(parent()->property("Message"));
}

double OrgDeepinLinglongTask1Adaptor::percentage() const
{
    // get the value of property Percentage
    return qvariant_cast< double >(parent()->property("Percentage"));
}

int OrgDeepinLinglongTask1Adaptor::state() const
{
    // get the value of property State
    return qvariant_cast< int >(parent()->property("State"));
}

void OrgDeepinLinglongTask1Adaptor::Cancel()
{
    // handle method call org.deepin.linglong.Task1.Cancel
    parent()->Cancel();
}


#include "moc_gen_org_deepin_linglong_task1.cpp"
