/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.deepin.linglong.Task1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef GEN_ORG_DEEPIN_LINGLONG_TASK1_H
#define GEN_ORG_DEEPIN_LINGLONG_TASK1_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.deepin.linglong.Task1
 */
class OrgDeepinLinglongTask1Interface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.deepin.linglong.Task1"; }

public:
    OrgDeepinLinglongTask1Interface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgDeepinLinglongTask1Interface();

    Q_PROPERTY(int Code READ code)
    inline int code() const
    { return qvariant_cast< int >(property("Code")); }

    Q_PROPERTY(QString Message READ message)
    inline QString message() const
    { return qvariant_cast< QString >(property("Message")); }

    Q_PROPERTY(double Percentage READ percentage)
    inline double percentage() const
    { return qvariant_cast< double >(property("Percentage")); }

    Q_PROPERTY(int State READ state)
    inline int state() const
    { return qvariant_cast< int >(property("State")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Cancel()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Cancel"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace deepin {
    namespace linglong {
      using Task1 = ::OrgDeepinLinglongTask1Interface;
    }
  }
}

#endif
