/****************************************************************************
** Meta object code from reading C++ file 'gen_org_deepin_linglong_packagemanager1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "gen_org_deepin_linglong_packagemanager1.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gen_org_deepin_linglong_packagemanager1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgDeepinLinglongPackageManager1Interface::qt_create_metaobjectdata<qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgDeepinLinglongPackageManager1Interface",
        "PruneFinished",
        "",
        "taskID",
        "QVariantMap",
        "result",
        "RequestInteraction",
        "QDBusObjectPath",
        "task",
        "messageID",
        "additionalMessage",
        "SearchFinished",
        "TaskAdded",
        "TaskRemoved",
        "Install",
        "QDBusPendingReply<QVariantMap>",
        "parameters",
        "InstallFromFile",
        "QDBusUnixFileDescriptor",
        "fd",
        "fileType",
        "options",
        "Permissions",
        "QDBusPendingReply<>",
        "Prune",
        "ReplyInteraction",
        "replies",
        "Search",
        "Uninstall",
        "Update",
        "Configuration"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PruneFinished'
        QtMocHelpers::SignalData<void(const QString &, const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'RequestInteraction'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, int, const QVariantMap &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Int, 9 }, { 0x80000000 | 4, 10 },
        }}),
        // Signal 'SearchFinished'
        QtMocHelpers::SignalData<void(const QString &, const QVariantMap &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'TaskAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'TaskRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'Install'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QVariantMap &)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 4, 16 },
        }}),
        // Slot 'InstallFromFile'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QDBusUnixFileDescriptor &, const QString &, const QVariantMap &)>(17, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 18, 19 }, { QMetaType::QString, 20 }, { 0x80000000 | 4, 21 },
        }}),
        // Slot 'Permissions'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(22, 2, QMC::AccessPublic, 0x80000000 | 23),
        // Slot 'Prune'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>()>(24, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'ReplyInteraction'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &)>(25, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 4, 26 },
        }}),
        // Slot 'Search'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QVariantMap &)>(27, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 4, 16 },
        }}),
        // Slot 'Uninstall'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QVariantMap &)>(28, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 4, 16 },
        }}),
        // Slot 'Update'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QVariantMap &)>(29, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 4, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Configuration'
        QtMocHelpers::PropertyData<QVariantMap>(30, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgDeepinLinglongPackageManager1Interface, qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgDeepinLinglongPackageManager1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>.metaTypes,
    nullptr
} };

void OrgDeepinLinglongPackageManager1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgDeepinLinglongPackageManager1Interface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PruneFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 1: _t->RequestInteraction((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3]))); break;
        case 2: _t->SearchFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 3: _t->TaskAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 4: _t->TaskRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 5: { QDBusPendingReply<QVariantMap> _r = _t->Install((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QVariantMap> _r = _t->InstallFromFile((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->Permissions();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QVariantMap> _r = _t->Prune();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->ReplyInteraction((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QVariantMap> _r = _t->Search((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<QVariantMap> _r = _t->Uninstall((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QVariantMap> _r = _t->Update((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackageManager1Interface::*)(const QString & , const QVariantMap & )>(_a, &OrgDeepinLinglongPackageManager1Interface::PruneFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackageManager1Interface::*)(const QDBusObjectPath & , int , const QVariantMap & )>(_a, &OrgDeepinLinglongPackageManager1Interface::RequestInteraction, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackageManager1Interface::*)(const QString & , const QVariantMap & )>(_a, &OrgDeepinLinglongPackageManager1Interface::SearchFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackageManager1Interface::*)(const QDBusObjectPath & )>(_a, &OrgDeepinLinglongPackageManager1Interface::TaskAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgDeepinLinglongPackageManager1Interface::*)(const QDBusObjectPath & )>(_a, &OrgDeepinLinglongPackageManager1Interface::TaskRemoved, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantMap*>(_v) = _t->configuration(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setConfiguration(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OrgDeepinLinglongPackageManager1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgDeepinLinglongPackageManager1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgDeepinLinglongPackageManager1InterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgDeepinLinglongPackageManager1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OrgDeepinLinglongPackageManager1Interface::PruneFinished(const QString & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgDeepinLinglongPackageManager1Interface::RequestInteraction(const QDBusObjectPath & _t1, int _t2, const QVariantMap & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void OrgDeepinLinglongPackageManager1Interface::SearchFinished(const QString & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgDeepinLinglongPackageManager1Interface::TaskAdded(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgDeepinLinglongPackageManager1Interface::TaskRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
