/****************************************************************************
** Meta object code from reading C++ file 'package_task.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../libs/linglong/src/linglong/package_manager/package_task.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'package_task.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8linglong7service11PackageTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto linglong::service::PackageTask::qt_create_metaobjectdata<qt_meta_tag_ZN8linglong7service11PackageTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "linglong::service::PackageTask",
        "StateChanged",
        "",
        "newState",
        "PercentageChanged",
        "newPercentage",
        "MessageChanged",
        "newMessage",
        "DataArrived",
        "arrived",
        "PartChanged",
        "fetched",
        "request",
        "CodeChanged",
        "newCode",
        "changePropertiesDone",
        "Cancel",
        "State",
        "Message",
        "Code",
        "Percentage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'PercentageChanged'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Signal 'MessageChanged'
        QtMocHelpers::SignalData<void(QString)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'DataArrived'
        QtMocHelpers::SignalData<void(uint)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 9 },
        }}),
        // Signal 'PartChanged'
        QtMocHelpers::SignalData<void(uint, uint)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 11 }, { QMetaType::UInt, 12 },
        }}),
        // Signal 'CodeChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'changePropertiesDone'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Cancel'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'State'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'Message'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'Code'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'Percentage'
        QtMocHelpers::PropertyData<double>(20, QMetaType::Double, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PackageTask, qt_meta_tag_ZN8linglong7service11PackageTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject linglong::service::PackageTask::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service11PackageTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service11PackageTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8linglong7service11PackageTaskE_t>.metaTypes,
    nullptr
} };

void linglong::service::PackageTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PackageTask *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->PercentageChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->MessageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->DataArrived((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->PartChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 5: _t->CodeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->changePropertiesDone(); break;
        case 7: _t->Cancel(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(int )>(_a, &PackageTask::StateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(double )>(_a, &PackageTask::PercentageChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(QString )>(_a, &PackageTask::MessageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(uint )>(_a, &PackageTask::DataArrived, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(uint , uint )>(_a, &PackageTask::PartChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)(int )>(_a, &PackageTask::CodeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PackageTask::*)()>(_a, &PackageTask::changePropertiesDone, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getPropertyState(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getPropertyMessage(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getPropertyCode(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->percentage(); break;
        default: break;
        }
    }
}

const QMetaObject *linglong::service::PackageTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *linglong::service::PackageTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service11PackageTaskE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    if (!strcmp(_clname, "Task"))
        return static_cast< Task*>(this);
    if (!strcmp(_clname, "TaskReporter"))
        return static_cast< TaskReporter*>(this);
    return QObject::qt_metacast(_clname);
}

int linglong::service::PackageTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void linglong::service::PackageTask::StateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void linglong::service::PackageTask::PercentageChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void linglong::service::PackageTask::MessageChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void linglong::service::PackageTask::DataArrived(uint _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void linglong::service::PackageTask::PartChanged(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void linglong::service::PackageTask::CodeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void linglong::service::PackageTask::changePropertiesDone()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
namespace {
struct qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t {};
} // unnamed namespace

template <> constexpr inline auto linglong::service::PackageTaskQueue::qt_create_metaobjectdata<qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "linglong::service::PackageTaskQueue",
        "taskDone",
        "",
        "taskID"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'taskDone'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PackageTaskQueue, qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject linglong::service::PackageTaskQueue::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>.metaTypes,
    nullptr
} };

void linglong::service::PackageTaskQueue::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PackageTaskQueue *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->taskDone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PackageTaskQueue::*)(const QString & )>(_a, &PackageTaskQueue::taskDone, 0))
            return;
    }
}

const QMetaObject *linglong::service::PackageTaskQueue::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *linglong::service::PackageTaskQueue::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8linglong7service16PackageTaskQueueE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int linglong::service::PackageTaskQueue::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void linglong::service::PackageTaskQueue::taskDone(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
