// This file is generated by tools/codegen.sh
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     RepositoryCacheMergedItem.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "linglong/api/types/v1/helper.hpp"

namespace linglong {
namespace api {
namespace types {
namespace v1 {
/**
* Merge multiple modules into one directory
*/

using nlohmann::json;

/**
* Merge multiple modules into one directory
*/
struct RepositoryCacheMergedItem {
/**
* binary module ostree commit hash
*/
std::optional<std::string> binaryCommit;
/**
* list of ostree commit hashes for each module
*/
std::vector<std::string> commits;
/**
* binary module ostree commit hash
*/
std::string id;
/**
* list of merged module names
*/
std::vector<std::string> modules;
/**
* human readable name
*/
std::optional<std::string> name;
};
}
}
}
}

// clang-format on
