// This file is generated by tools/codegen.sh
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     DialogHandShakePayload.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "linglong/api/types/v1/helper.hpp"

namespace linglong {
namespace api {
namespace types {
namespace v1 {
using nlohmann::json;

struct DialogHandShakePayload {
/**
* Minimum protocol version supported by the client.
* Current version is 1.0.
* 1.0 behavior: dialog MUST report self minimal protocol version at program start.
* if client not reported protocol version, caller SHOULD kill dialog after timeout.
* then, caller will send a directory permission request to dialog. if timeout,
* client SHOULD exit immediately. after that, caller will wait for dialog exit
* and use dialog exit code to know user allow the request or not. dialog MUST exit
* with 0, any other value means user reject the request or dialog internal error and
* dialog could output any message to stderr.
*/
std::string version;
};
}
}
}
}

// clang-format on
