// This file is generated by tools/codegen.sh
// DO NOT EDIT IT.

// clang-format off

//  To parse this JSON data, first install
//
//      json.hpp  https://github.com/nlohmann/json
//
//  Then include this file, and then do
//
//     ExtensionImpl.hpp data = nlohmann::json::parse(jsonString);

#pragma once

#include <optional>
#include <nlohmann/json.hpp>
#include "linglong/api/types/v1/helper.hpp"

#include "linglong/api/types/v1/DeviceNode.hpp"

namespace linglong {
namespace api {
namespace types {
namespace v1 {
using nlohmann::json;

struct ExtensionImpl {
/**
* device nodes to be mounted
*/
std::optional<std::vector<DeviceNode>> deviceNodes;
/**
* environment variables to set when extension effects
*/
std::optional<std::map<std::string, std::string>> env;
/**
* extra ld search path
*/
std::optional<std::vector<std::string>> libs;
};
}
}
}
}

// clang-format on
